#!/tvbin/tivosh
#
# Tcl script to print out a list of tystreams that have been
# recorded. This comes from the TiVo web server written by
# Stephen Rothwell (sfr@linuxcare.com.au)
#
# It works on 2.x, but you may need to change the /Recording/NowShowing
# line about 10 lines down to say /Recording/Complete if you have
# a 1.3 system.
#
source $tcl_library/tv/log.tcl
source $tcl_library/tv/mfslib.tcl
source $tcl_library/tv/sendkey.tcl

proc action_NowShowing {} {
    puts "Now Showing"
    puts "-----------"
    
    ForeachMfsFile fsid name type  "/Recording/NowShowing" "" { 
	if {$type == "tyDb"} {
	    set db [dbopen]
	    transaction { 
		set tystreams ""
		set foundnotshow true
		set foundnotpart true
		if {[catch {set obj [db $db openidconstruction $fsid 10]}] == 0} {
		    foreach attr [dbobj $obj attrs] {
			if {$attr == "Showing" && $foundnotshow} {
			    set foundnotshow false
			    set foundnotprog true
			    if { [dbobj $obj attrtype $attr] == "object" } {
				foreach obj2 [dbobj $obj gettarget $attr] {
				    regexp {([0-9]*)/(.*)} $obj2 junk part1 part2
				    set obj3 [db $db openidconstruction $part1 $part2]
				    foreach attr2 [dbobj $obj3 attrs] {
					if {$attr2 == "Program" && $foundnotprog} {
					    set foundnotprog false
					    if { [dbobj $obj3 attrtype $attr2] == "object" } { 
						foreach obj4 [dbobj $obj3 gettarget $attr2] { 
						    regexp {([0-9]*)/(.*)} $obj4 junk part1 part2
						    set obj5 [db $db openidconstruction $part1 $part2]
						    foreach attr3 [dbobj $obj5 attrs] {
							if {$attr3 == "Title" || 
							$attr3 == "Description" || 
							$attr3 == "EpisodeTitle" } {
							    foreach obj6 [dbobj $obj5 get $attr3] { 
								puts -nonewline $attr3 
								puts -nonewline ": " 
								puts $obj6
								if {$attr3 == "Title"} {
								    set title $obj6
								}
							    }
							}
						    }
						}
					    }
					}
				    }
				}
			    }
			}
			if {$attr == "Part" && $foundnotpart} {
			    set foundnotpart false
			    if { [dbobj $obj attrtype $attr] == "object" } {
				foreach obj2 [dbobj $obj gettarget $attr] {
				    regexp {([0-9]*)/(.*)} $obj2 junk part1 part2
				    set obj3 [db $db openidconstruction $part1 $part2]
				    foreach attr2 [dbobj $obj3 attrs] {
					if {$attr2 == "File"} {
					    foreach obj4 [dbobj $obj3 get $attr2] { 
						append tystreams "$obj4"
					    }
					}
				    }
				}
			    }
			}
			if {$attr == "RecordQuality"} {
			    if { [dbobj $obj get $attr] == 0 } {
				set quality "TiVoBasic"
			    } else {
				set quality "TiVoBest"
			    }
			}
		    }
		}
		regsub -all " " $title "%20" newtitle
		puts -nonewline "TyStream:"
		puts $tystreams
		puts ""
	    }
	}
    }
}

action_NowShowing
