#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "../../hdr/new_tystream.h"
#include "../../hdr/libplayit.h"
#include "../../hdr/nplayitsam.h"
#include "../../hdr/cuts.h"

/* External things */

extern GtkWidget *text1;
extern GtkWidget *dialog1;
extern GtkWidget *entry1;
extern GtkWidget *entry2;
extern GtkWidget *label2;
extern GtkWidget *fileselection1;
extern GtkObject *adj1;

/* Global things */
int Debug= 0;			/* Turn on debugging messages */
int SeparateFiles= 0;		/* Place material into several files */

/* Static things */
static char printfbuf[1024];	/* Buffer for our printf */
static int Was_a_jump= 0;	/* A jump before playing this group? */
static int fd= 0;		/* File descriptor of incoming stream */

static u_int32_t numchunks;	/* Number of chunks in input file */
static u_int32_t Position= 0;	/* Number of chunks in input file */
static int Paused= 0;		/* If 1, we are paused */
static int Recording= 0;	/* If >1, we are recording */
static int file_has_data= 0;	/* Used to determine if to separate files */
static guint Playregister= 0;	/* Have we registered the play fn? */

static FILE *afile, *vfile;	/* Recording streams, i.e Save As streams */
				/* Streams connected to play pipes */
static FILE *audio_file=NULL, *video_file=NULL;
static char *Filename= NULL;	/* Output filename supplied by user */
static gchar *Hostname= NULL;	/* The hostname supplied by the user */

#define AUDIO_PROG "mpg123 - 2> /dev/null"
#define VIDEO_PROG "mpeg2dec -o x11 2> /dev/null"

/*
 * Open the output devices used for playback. Returns 1 if ok, 0 if an error.
 */
int open_playback_devices(FILE ** audio, FILE ** video)
{
  *audio= popen(AUDIO_PROG, "w");
  if (*audio == NULL) {
    printf("Failed to start audio program %s\n", AUDIO_PROG);
    return (0);
  }
  *video= popen(VIDEO_PROG, "w");
  if (*video == NULL) {
    printf("Failed to start video program %s\n", VIDEO_PROG);
    return (0);
  }
  return (1);
}

/* Open out Save as output files.
 * Returns 1 if ok, 0 on error
 */
int gtk_open_files(char *basename, int count)
{
#define OPENLEN 100
  char abuf[OPENLEN], vbuf[OPENLEN];

  if (count == 0) {
    snprintf(abuf, OPENLEN - 6, "%s.mpa", basename);
    snprintf(vbuf, OPENLEN - 6, "%s.mpv", basename);
  } else {
    snprintf(abuf, OPENLEN - 6, "%s%d.mpa", basename, count);
    snprintf(vbuf, OPENLEN - 6, "%s%d.mpv", basename, count);
  }

  afile= fopen(abuf, "wb");
  if (afile == NULL) { printf("Cannot write file %s\n", abuf); return (0); }
  vfile= fopen(vbuf, "wb");
  if (vfile == NULL) { printf("Cannot write file %s\n", vbuf); return (0); }
  return (1);
}


/*
 * Unfortunately, we can't use the split_stream() function from the
 * libplayit.a because it loops and stops the GTK window from being
 * displayed. This does the same thing, but is called repetetively until the
 * files are written.
 */

gint gtk_split_stream(gpointer G)
{
  int i, c, cutend;
  int is_splitpoint;
  struct new_ty_node *H;
  gfloat value;

  /* The end is nigh, as we are out of chunks to record */
  if (Position >= numchunks) {
    printf("Files saved\n");
    fclose(afile); fclose(vfile);
    if (Filename) { free(Filename); Filename=NULL; }
    Position= 0; Paused=0; Was_a_jump= 1;
    gtk_idle_remove(Recording); Recording= 0;
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj1), 0);
    return (FALSE);
  }


  /* Update the chunk number and also the scrollbar */
  snprintf(printfbuf, 1023, "%d", Position);
  gtk_label_set_text((GtkLabel *) label2, printfbuf);

  /* Calculate the scroll bar's new value */
  value= (float)Position/(float)numchunks * GTK_ADJUSTMENT(adj1)->upper;
  gtk_adjustment_set_value(GTK_ADJUSTMENT(adj1), value);

  

  /* Determine what to do with this chunk number */
  i= find_chunk_in_cutlist(Position, &cutend, &is_splitpoint);

  switch (i) {
  case IS_INSIDE_CUT:
  case IS_CUT_START:
    Position= cutend; return (TRUE);
  }

  /* We will try to read c in */
  c= Position;

  if (Debug >= 1) fprintf(stderr,"Getting GOP start at chunk %d\n", Position);
  H= read_group(c, &Position);
  if (H == NULL) {
    gtk_idle_remove(Recording); Recording= 0; return (FALSE);
  }

  /*
   * First open new files if required, but not if the current file is still
   * empty.
   */
  if  ((SeparateFiles != 0) && (is_splitpoint == 1) && (file_has_data == 1)) {
    SeparateFiles++;
    if (afile != NULL) fclose(afile);
    if (vfile != NULL) fclose(vfile);
    if (gtk_open_files(Filename, SeparateFiles) == 0) {
      printf("Could not open output files!\n"); return(FALSE);
    }
    file_has_data= 0;
  }

  /* Kill the B frames if this is the end of a cut */
  if  (i == IS_CUT_END) {
    remove_B_frames(H);
    if (Debug >= 1) {
      fprintf(stderr, "After B-removal, "); print_newtylist(H, c);
    }
  }

  /* Now write the thing out */
  if (play_group(H, afile, vfile) == 0) return(FALSE);

  file_has_data= 1;
  return (TRUE);
}

/* Replacement for printf that allows us
 * to print to the text area.
 */

int printf(char const *fmt, ...)
{
  va_list ap;
  int ret;

  va_start(ap, fmt);
  ret= vsnprintf(printfbuf, 1023, fmt, ap);
  va_end(ap);

  if (ret > 0)
    gtk_text_insert(GTK_TEXT(text1), NULL, NULL, NULL, printfbuf, -1);
  return (ret);
}


/* This function plays the group at Position. */
gint play_one_group(gpointer G)
{
  int c;
  struct new_ty_node *H;

  c= Position;
  snprintf(printfbuf, 1023, "%d", Position);
  gtk_label_set_text((GtkLabel *) label2, printfbuf);

  H= read_group(c, &Position);
  if (H == NULL) return (FALSE);

  /* Before we play it, we remove B-frames if it was a jump */
  if (Was_a_jump == 1) {
    remove_B_frames(H);
    if (Debug >= 1) {
      fprintf(stderr, "After B-removal, ");
      print_newtylist(H, c);
    }
  }

  Was_a_jump= 0;
  if (play_group(H, audio_file, video_file) == 0) return(FALSE);

  /* Check we don't get too close to the end */
  if (Position >= numchunks) {
    Position= numchunks - 5; Was_a_jump= 1; Paused= 1;
  }
  return (TRUE);
}



void on_open1_activate(GtkMenuItem * menuitem,
		  gpointer user_data)
{
  GtkWidget *FSIDdialog;
  FSIDdialog= create_dialog1();
  if (Hostname) gtk_entry_set_text((GtkEntry *)entry1, Hostname);
  gtk_widget_show(FSIDdialog);
}


void
on_save1_activate(GtkMenuItem * menuitem,
		  gpointer user_data)
{
  GtkWidget *Savedialog;
  Savedialog= create_fileselection1();
  gtk_widget_show(Savedialog);
}


void
on_quit1_activate(GtkMenuItem * menuitem,
		  gpointer user_data)
{
  exit(0);
}


void
on_button1_clicked(GtkButton * button,
		   gpointer user_data)
{
  printf("You've found one of the two unused buttons in this program!\n");
}


void
on_button5_clicked(GtkButton * button,
		   gpointer user_data)
{
  if (Recording) return;
  add_cut_start(Position);
  printf("Cut starting at chunk %d\n", Position);
}


void
on_button12_clicked(GtkButton * button,
		    gpointer user_data)
{
  gtk_object_destroy(GTK_OBJECT(user_data));
}

void
on_button11_clicked(GtkButton * button,
		    gpointer user_data)
{
  gchar *FSID= NULL;
  int c;
  char *fname;

  if (Recording) return;
  if (Hostname) { free(Hostname); Hostname=NULL; }
  Hostname= strdup(gtk_editable_get_chars((GtkEditable *) entry1, 0, -1));
  FSID= gtk_editable_get_chars((GtkEditable *) entry2, 0, -1);
  if (dialog1) gtk_widget_destroy(dialog1);

  /* If existing open stream, close it */
  if (audio_file) on_close_stream_from_tivo1_activate(NULL, NULL);

  /* Concatenate the hostname and FSID list */
  fname= (char *)malloc(strlen(Hostname) + strlen(FSID) + 3);
  if (fname == NULL) { fprintf(stderr, "Cannot malloc!\n"); return; }
  strcpy(fname, Hostname);
  strcat(fname, ":");
  strcat(fname, FSID);


  fd= open_new_tystream(fname, &numchunks, &c);
  if (fd == 0) { printf("Can't open %s\n", fname); free(fname); return; }
  printf("Stream has %d chunks\n", numchunks);

  if (open_playback_devices(&audio_file, &video_file) == 0) exit(1);

  /* Register the play_one_group function */
  if ((Playregister == 0) && (fd != 0))
    Playregister= gtk_idle_add(play_one_group, NULL);
}

/**** Play button ****/
void on_button3_clicked(GtkButton * button,
		   gpointer user_data)
{
  if (Recording) return;

  /* If Paused, call the function only once, i.e a step function */
  if (Paused == 1) {
    play_one_group(NULL);
  } else { 			/* Register the play_one_group function */
    if ((Playregister == 0) && (fd != 0))
      Playregister= gtk_idle_add(play_one_group, NULL);
  }
}


/**** Pause button ****/
void on_button2_clicked(GtkButton * button,
		   gpointer user_data)
{
  if (Recording) return;
  Paused= 1 - Paused;		/* Toggle the paused status */
  if (Paused == 1) {
    /* Deregister the play_one_group function */
    if (Playregister) gtk_idle_remove(Playregister);
    Playregister= 0;
  } else {
    if ((Playregister == 0) && (fd != 0))
      Playregister= gtk_idle_add(play_one_group, NULL);
  }
}


void
on_button4_clicked(GtkButton * button,
		   gpointer user_data)
{
  printf("You've found one of the two unused buttons in this program!\n");
}


void
on_button6_clicked(GtkButton * button,
		   gpointer user_data)
{
  int cstart;
  if (Recording) return;
  if ((cstart= add_cut_end(Position, CMD_CEND)) == -1) {
    printf("There is no pending cut to end here!\n"); return;
  }
  printf("New cut from %d to %d\n", cstart, Position);
}


void
on_button7_clicked(GtkButton * button,
		   gpointer user_data)
{
  int cstart;

  if (Recording) return;
  if ((cstart= add_cut_end(Position, CMD_SPLITPOINT)) == -1) {
    printf("Error setting splitpoint here\n"); return;
  }
  printf("New splitpoint at %d\n", Position);
  SeparateFiles=1;
}


void
on_button8_clicked(GtkButton * button,
		   gpointer user_data)
{
  int cstart;

  if (Recording) return;
  if ((cstart= add_cut_end(Position, CMD_CENDSPLIT)) == -1) {
    printf("There is no pending cut to end here!\n");
    return;
  }
  printf("New cut and splitpoint from %d to %d\n", cstart, Position);
  SeparateFiles=1;
}


void
on_button9_clicked(GtkButton * button,
		   gpointer user_data)
{
  if (Recording) return;
  gtk_text_backward_delete((GtkText *) text1, 10000);
  printf("\nHere is the cut list:\n");
  list_cuts(Position);
}


void
on_button10_clicked(GtkButton * button,
		    gpointer user_data)
{
  if (Recording) return;
  printf("Killing all cuts\n");
  kill_all_cuts();
}

/**** OK button in Save File Dialog box ****/
void
on_ok_button1_clicked(GtkButton * button,
		      gpointer user_data)
{
  if (Recording) return;
  if (Filename) { free(Filename); Filename=NULL; }
  Filename= strdup(gtk_file_selection_get_filename
    					(GTK_FILE_SELECTION(fileselection1)));
  if (fileselection1) gtk_widget_destroy(fileselection1);

  if (Filename == NULL) {
    printf("No filename supplied, cannot save program\n"); return;
  }

  /* Open the files */
  if (gtk_open_files(Filename, SeparateFiles) == 0) {
    printf("Could not open output files!\n"); return;
  }

  /* Register the gtk_split_stream() function so that it is called */
  if (Playregister) { gtk_idle_remove(Playregister); Playregister=0; }
  Recording= gtk_idle_add(gtk_split_stream, NULL);
  printf("\nStarting to record the program ...\n");
  Position= 0;
}


/**** Cancel button in Save File Dialog box ****/
void on_cancel_button1_clicked(GtkButton * button,
			  gpointer user_data)
{
  if (fileselection1) gtk_widget_destroy(fileselection1);
}


void on_close_stream_from_tivo1_activate(GtkMenuItem * menuitem,
				    gpointer user_data)
{
  if (Recording) return;
  if (audio_file==NULL) return;

  fclose(audio_file); fclose(video_file);
  if (Filename) { free(Filename); Filename=NULL; }

  kill_all_cuts();
  if (Playregister) gtk_idle_remove(Playregister);
  gtk_label_set_text((GtkLabel *) label2, "");

  Playregister= 0; Was_a_jump= 0; fd= 0;
  numchunks= 0; Position= 0; Paused= 0; Recording= 0;
  audio_file=NULL;
}

void scrollbar_value_changed(GtkWidget * widget,
			GdkEventButton * event,
			gpointer user_data)
{
  GtkAdjustment *adj;
  gfloat fract;

  if (Recording) return;
  if (numchunks == 0) return;

  /*
   * Get the fractional position from the widget, assume that lower is 0.0.
   */
  adj= (GtkAdjustment *) user_data;
  fract= adj->value / adj->upper;

  /* Set new Position. */
  if ((fract >= 0.0) && (fract <= 1.0)) {
    Position= (int)((float)numchunks * fract);
    snprintf(printfbuf, 1023, "%d", Position);
    gtk_label_set_text((GtkLabel *) label2, printfbuf);
    Was_a_jump= 1;
  }
  /* Play a group if paused */
  if (Paused == 1) play_one_group(NULL);
}
