/* new_tystream.c: Deal with the new tystream format. $Revision: 1.10 $
 *
 * (c) 2002, Warren Toomey wkt@tuhs.org.
 * 
 * Released under the Gnu General Public License version 2
 */

#include "defines.h"
#include <sys/types.h>
#include <stdio.h>
#include "new_tystream.h"
#include "old_tystream.h"
#include "libplayit.h"


extern int Debug;

#if 0
/* Given a stream name, open that stream and return the
 * number of records in the stream, and the size of the
 * stream in kilobytes. Returns a positive
 * stream descriptor if ok, or -1 if failure.
 *
 * The name can be a Unix filename, a set of FSIDs or a
 * machine:stream_id combination.
 */
int open_new_tystream(char *name, u_int32_t *num_recs, u_int32_t *size_in_k)
{
}

/* Close an open tystream. Returns 0 if ok, -1 if failure */
int close_new_tystream(int fd)
{
}

/* Read the given record from the tystream. The table pointer
 * will be allocated internally to an array holding the
 * entries, and these will point to buffers for the data.
 * The index number is >=0 and < the original num_recs.
 * Returns the next record number in the stream, 0 on end of
 * the stream, or -1 on error.
 */
int read_new_table(int fd, u_int32_t index, struct new_ty_node **table, u_int32_t *num_entries)
{
}

/* Free a table of entries and the data to which they point.
 * Returns 0 if ok, -1 on error.
 */
int free_new_table(struct new_ty_node *table, u_int32_t num_entries)
{
}
#endif



/* Given a group of pictures, find the first B frame and the first I
 * frame. Remove the two B-frames after the I-frame. Mark the GOP
 * as broken. Remove audio frames within the B-frame time range.
 */

#define GOP_FLAG_BROKEN  0x20
#define GOP_FLAG_CLOSED  0x40

void remove_B_frames(struct new_ty_node *G)
{
  u_int32_t lo_vidstamp=0, hi_vidstamp=0;
  struct new_ty_node *This, *That, *Last=NULL;

  if (G==NULL) return;
  for (This=G; This!=NULL; Last=This, This=This->next) {
    switch(This->type) {
      case GOP_ENTRY:
	/* Mark the GOP as closed and broken */
        This->buf[7] |= (GOP_FLAG_BROKEN | GOP_FLAG_CLOSED);
	break;
      case I_ENTRY:

	/* Use the I frame's timestamp as the top one */
	hi_vidstamp= This->timestamp;

	/* Firstly, set This so that This->next is a B-frame */
        for (;(This!=NULL) && (This->next->type!=B_ENTRY); This=This->next) ;

	/* This now is the last video record before the B-frame. */
	/* Walk until we find either a P-frame or audio */
	for (That=This; That!=NULL; That=That->next){
	  if (That->type==P_ENTRY) break;
	  if (That->type==AUDIO_ENTRY) break;
	  if ((lo_vidstamp==0) && (That->type==B_ENTRY))
	     lo_vidstamp= That->timestamp;
        }

	/* That is now past the B-frames, jump them. */
        This->next= That; break;

      case AUDIO_ENTRY:

	/* Remove this audio entry if it is within range */
	if ((This->timestamp>=lo_vidstamp)&&(This->timestamp<hi_vidstamp)) {
	  Last->next= This->next;
	  /* Move This backwards to Last, so when we loop forwards we don't
	   * bring Last up to the missing record.
	   */
	  This=Last;
        }
    }
  }
  if (Debug>=2) {
    printf("Group with B frames removed, ");
    print_newtylist(G, 0);
  }
}

/* Play the audio and video data to the playback devices.
 * Returns 1 if successful, 0 if errors.
 */
int play_group(struct new_ty_node *G, FILE *audio, FILE *video)
{
  struct new_ty_node *audnode, *vidnode;

  if ((audio==NULL) || (video==NULL) || (G==NULL)) return(0);

  /* Walk the audio pointer down until we hit an audio node */
  audnode= vidnode= G;

  while ((audnode!=NULL) && (audnode->type!=AUDIO_ENTRY))
    audnode= audnode->next;

   if (Debug==1) printf("About to play data in play_group()\n");

  /* Now loop playing data until audnode is NULL and
   * the vidnode points to NULL or an audio node.
   */
  while (1) {
    if ((audnode==NULL) && (vidnode==NULL)) break;
    if ((audnode==NULL) && (vidnode->type==AUDIO_ENTRY)) break;

    /* Play an audio entry */
    if (audnode!=NULL) {
       fwrite(audnode->buf, 1, audnode->size, audio);
       if (Debug==1) printf("Sent %d to audio device\n",audnode->size);
       audnode= audnode->next;
    }

    /* Play a video entry */
    if (vidnode!=NULL) {
       fwrite(vidnode->buf, 1, vidnode->size, video);
       if (Debug==1) printf("Sent %d to video device\n",vidnode->size);
       vidnode= vidnode->next;
    }
  }
  return(1);
}
