/*
 * old_tystream.c: Deal with native TiVo tystreams. $Revision: 1.4 $
 * 
 * (c) 2002, Warren Toomey wkt@tuhs.org.
 * 
 * Released under the Gnu General Public License version 2
 */

#include "defines.h"
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "new_tystream.h"
#include "old_tystream.h"

extern int Debug;

/*
 * Local variables. We should set up an array of these so that we can open >1
 * input file, but for now one file is enough.
 */

static int skip_two_chunk=0;	/* Need to skip 2 chunks every 4096? */

/*
 * Given a stream name, open that stream and return the number of 128K chunks
 * in the stream. Returns a positive stream descriptor if ok, or -1 if
 * failure.
 * 
 * The name can only be a Unix filename.
 */
int open_old_local_tystream(char *name, u_int32_t * num_chunks)
{
  struct stat statbuf;
  int s, fd;
  u_int8_t buf[4];

  if ((name == NULL) || (num_chunks == NULL)) return (-1);

  if ((fd=open(name, O_RDONLY)) == -1) return (-1);

  if (fstat(fd, &statbuf) == -1) return (-1);

  /*
   * Read the first chunk. If it contain F5 46 XX BD, then it's an invalid
   * chunk. We will need to skip 2 chunks out of every 4096 chunks.
   */
  if (read(fd, buf, 4) != 4) return (-1);
  if ((buf[0] != 0xf5) && (buf[1] != 0x46) && (buf[3] != 0xbd))
    skip_two_chunk=0;
  else
    skip_two_chunk=1;

  *num_chunks=statbuf.st_size / CHUNK_SIZE;

  /* Reduce the size if we need to skip 2 chunks out of every 4096 chunks. */
  if (skip_two_chunk==1) {
    s= *num_chunks/4096; s= 2*s + 1;
    *num_chunks -= s;
  }
  
  if (Debug>=4) printf("SET num_chunks to %d\n",*num_chunks);
  return (fd);
}

/* Close an open tystream. Returns 0 if ok, -1 if failure */
int close_old_local_tystream(int fd)
{
  return (close(fd));
}

/*
 * Read the given chunk from the tystream and store it in the given 128K buf.
 * The buf must already exist. The chunknum is >=0 and < the original
 * num_chunks. Returns 1 if ok, 0 on error.
 */
int read_old_local_chunk(int fd, u_int8_t * buf, u_int32_t chunknum)
{
  int i;
  int numleft=CHUNK_SIZE, numread;
  u_int32_t c,s;

  /* Seek to the chunk's position in the file.
   * If this stream hasn't had the unwanted chunks
   * removed, then we have to skip 2 chunks out of
   * every 4096.
   */
  c= chunknum;

  if (skip_two_chunk==1) {
    /* Work out which logical stream we're in */
    s=c/4094;

    /* Add 2*s+1 */
    c+= 2*s + 1;
  }

  i=lseek(fd, CHUNK_SIZE * c, SEEK_SET);
  if (i == -1) return (0);

  /* Loop until we get CHUNK_SIZE bytes of data */
  while (numleft > 0) {
    numread=read(fd, buf, numleft);
    if (numread < 1) return (0);
    numleft -= numread;
    buf += numread;
  }
  return (1);
}
