#include "defines.h"
#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include "new_tystream.h"
#include "old_tystream.h"
#include "nplayitsam.h"
#include "cuts.h"
#include "libplayit.h"

extern int Debug;		/* Turn on debugging messages */
extern int SeparateFiles;	/* Place material into several files */
extern u_int32_t TotalChunks;	/* Total number of chunks in the file */
static FILE *afile = NULL, *vfile = NULL;

/* Returns 1 if ok, 0 on error */
int open_files(char *abasename, char *vbasename, int count)
{
#define OPENLEN 100
  char abuf[OPENLEN], vbuf[OPENLEN];
  char *newaname, *newvname;

  if (count == 0) {
    newaname = abasename; newvname = vbasename;
  } else {
    snprintf(abuf, OPENLEN - 2, "%s%d", abasename, count);
    snprintf(vbuf, OPENLEN - 2, "%s%d", vbasename, count);
    newaname = abuf; newvname = vbuf;
  }

  afile = fopen(newaname, "wb");
  if (afile == NULL) { printf("Cannot write file %s\n", newaname); return (0); }
  vfile = fopen(newvname, "wb");
  if (vfile == NULL) { printf("Cannot write file %s\n", newvname); return (0); }
  return (1);
}


int split_stream(char *audio_name, char *video_name)
{
  int i, c, chunk, cutend;
  int is_splitpoint;
  struct new_ty_node *H;
  int file_has_data = 0;
  int groups = 0;

  if (open_files(audio_name, video_name, SeparateFiles) == 0) return(-1);
  if (SeparateFiles != 0) SeparateFiles++;

  /* Loop over all the chunks */
  for (chunk = 0; chunk < TotalChunks;) {

    /* Determine what to do with this chunk number */
    i = find_chunk_in_cutlist(chunk, &cutend, &is_splitpoint);

    switch (i) {
    case IS_INSIDE_CUT:
    case IS_CUT_START:
      chunk = cutend; continue;
    }

    /* We will try to read c in */
    c = chunk;

    if (Debug >= 1) printf("Getting GOP starting at chunk %d\n", c);
    H = read_group(c, &chunk);
    if (H == NULL) break;

    /*
     * First open new files if required, but not if the current file is still
     * empty.
     */
    if ((SeparateFiles != 0) && (is_splitpoint == 1) && (file_has_data == 1)) {
      if (afile != NULL) fclose(afile);
      if (vfile != NULL) fclose(vfile);
      if (open_files(audio_name, video_name, SeparateFiles) == 0) exit(1);
      SeparateFiles++;
      file_has_data = 0;
    }
    /* Kill the B frames if this is the end of a cut */
    if (i == IS_CUT_END) {
      remove_B_frames(H);
      if (Debug >= 1) {
	printf("After B-removal, "); print_newtylist(H, c);
      }
    }
    /* Now write the thing out */
    if (play_group(H, afile, vfile) == 0) exit(1);

    file_has_data = 1; groups++;
    if ((groups != 0) && ((groups % 20) == 0))
      printf("%3d groups written\n", groups);
  }
  return (0);
}
