#include "defines.h"
#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include "new_tystream.h"
#include "old_tystream.h"
#include "nplayitsam.h"
#include "cuts.h"
#include "libplayit.h"
#include "proto.h"

int Debug= 0;				/* Turn on debugging messages */
int SeparateFiles= 0;			/* Place material into several files */

static u_int32_t numchunks;		/* Number of chunks in input file */
					/* Filenames when we record */

					/* Output file names and streams */
static char *aud_filename= NULL, *vid_filename = NULL;
static FILE *audio_file, *video_file;


static void usage(void)
{
  printf("playitsam [options] <infile> [audio_out video_out]\n");
  printf("options:\n");
  printf("  -v                 verbose\n");
  printf("  -s                 split into separate files\n");
  printf("  -h                 help\n");
  exit(1);
}



static void edit_stream()
{
  int c, chunk, cmd;
  int was_a_jump= 0;
  int cstart;
  int paused= 0;
  struct new_ty_node *H;

  /*
   * Start from the first chunk and play the chunk. Then try to get commands
   * from the user.
   */
  for (chunk= 0; chunk < numchunks;) {
    c= chunk;
    printf("\rChunk %d", c); printf("                     ");
    fflush(stdout);

    /*
     * If we are paused and no jump, do nothing but get a new command.
     */
    if ((paused == 1) && (was_a_jump == 0)) goto here;	/* Yuk a goto */

    H=read_group(c, &chunk);
    if (H == NULL) break;

    /* Before we play it, we remove B-frames if it was a jump */
    if (was_a_jump==1) {
      remove_B_frames(H);
      if (Debug >= 1) { printf("After B-removal, "); print_newtylist(H, c); }
    }
    was_a_jump= 0;
    if (play_group(H, audio_file, video_file) == 0) exit(1);

here:
    if (paused == 1) usleep(100000);

    /* Obtain a command */
    cmd= read_a_command();
    switch (cmd) {
    case CMD_PAUSE: paused= 1 - paused; break;
    case CMD_START: chunk= 0; was_a_jump= 1; break;
    case CMD_FWDFRAME: was_a_jump=2; break;
    case CMD_BACKFRAME: chunk -= 7; was_a_jump= 1; break;
    case CMD_FASTF1: chunk += 5; was_a_jump= 1; break;
    case CMD_FASTF2: chunk += 25; was_a_jump= 1; break;
    case CMD_FASTF3: chunk += 125; was_a_jump= 1; break;
    case CMD_FASTF4: chunk += 500; was_a_jump= 1; break;
    case CMD_FASTF5: chunk += 2500; was_a_jump= 1; break;
    case CMD_FASTF6: chunk += 12500; was_a_jump= 1; break;
    case CMD_FASTB1: chunk -= 10; was_a_jump= 1; break;
    case CMD_FASTB2: chunk -= 25; was_a_jump= 1; break;
    case CMD_FASTB3: chunk -= 125; was_a_jump= 1; break;
    case CMD_FASTB4: chunk -= 500; was_a_jump= 1; break;
    case CMD_FASTB5: chunk -= 2500; was_a_jump= 1; break;
    case CMD_FASTB6: chunk -= 12500; was_a_jump= 1; break;

    case CMD_QUIT: exit(0);

    case CMD_CSTART:
      add_cut_start(c);
      fprintf(stderr, "\r\nCut starting at chunk %d\r\n", c);
      break;

    case CMD_CENDSPLIT:
    case CMD_CEND:
      if ((cstart= add_cut_end(c,cmd)) == -1) {
	fprintf(stderr, "\r\nThere is no pending cut to end here\r\n");
	break;
      }
      fprintf(stderr, "\r\nNew cut from %d to %d\r\n", cstart, c);
      break;

    case CMD_SPLITPOINT:
      if ((cstart= add_cut_end(c,cmd)) == -1) {
	fprintf(stderr, "\r\nError setting splitpoint here\r\n");
	break;
      }
      fprintf(stderr, "\r\nNew splitpoint at %d\r\n", c);
      break;

    case CMD_KILLALL:
      fprintf(stderr, "\r\nKilling all cuts\r\n");
      kill_all_cuts();
      break;

    case CMD_LISTCUTS:
      fprintf(stderr, "\r\nHere is the list of cuts:\r\n");
      list_cuts(c);
      break;

    case CMD_RECORD:
      if ((aud_filename == NULL) || (vid_filename == NULL)) {
	printf("\r\nYou can't record as you didn't name output files!\r\n");
	break;
      }
      fclose(audio_file); fclose(video_file);
      reset_terminal(0); printf("\n");
      split_stream(aud_filename, vid_filename);
      exit(0);
    }

    /* Keep the chunk numbers sane */
    if (chunk == 0) { chunk=c; paused=1; }
    if ((chunk>0) && (chunk >= numchunks))
	{ chunk= numchunks - 5; was_a_jump= 1; paused=1; }
    if (chunk < 0)
		{ chunk= 0; was_a_jump= 1; }
  }
}


int main(int argc, char *argv[])
{
  char *fname;
  extern int optind;
  int i, c;

  while ((c= getopt(argc, argv, "vsh")) != -1) {
    switch (c) {
    case 'v': Debug++; break;
    case 's': SeparateFiles++; break;
    case 'h':
    default: usage();
    }
  }

  argc -= optind; argv += optind;

  if (argc < 1) usage();
  fname= argv[0];

  if (argc == 3) {
    aud_filename= argv[1]; vid_filename= argv[2];
    printf("Wohoo, output files!\n");
  }

  i=open_new_tystream(fname, &numchunks, &c);
  if (i==0) { printf("Can't open %s\n", fname); exit(1); }
  printf("Stream has %d chunks\n", numchunks);

  if (open_playback_devices(&audio_file, &video_file) == 0) exit(1);

  /* Set standard input to non-blocking mode */
  set_nonblocking(0);

  edit_stream();
  return (0);
}
