/*
 * unixcli.c: Code only used on Unix/Linux for command-line editing.
 * $Revision: 1.6 $
 * 
 * (c) 2002, Warren Toomey wkt@tuhs.org.
 * 
 * Released under the Gnu General Public License version 2
 */

#include "defines.h"
#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include "nplayitsam.h"

static struct termios oldtios, newtios;         /* Console settings for raw */
                                                /* and cooked modes */

#define AUDIO_PROG "mpg123 - 2> /dev/null"
#define VIDEO_PROG "mpeg2dec -o x11 2> /dev/null"

/* Open the output devices used for playback. Returns
 * 1 if ok, 0 if an error.
 */
int open_playback_devices(FILE **audio, FILE **video)
{
  *audio= popen(AUDIO_PROG, "w");
  if (*audio==NULL) { 
    fprintf(stderr, "Failed to start audio program %s\n", AUDIO_PROG);
    return(0);
  }
  *video= popen(VIDEO_PROG, "w");
  if (*video==NULL) { 
    fprintf(stderr, "Failed to start video program %s\n", VIDEO_PROG);
    return(0);
  }
  return(1);
}


/* Set a fd into nonblocking mode */
void set_nonblocking(int fd)
{
  int val;

  if ((val= fcntl(fd, F_GETFL, 0)) == -1) return;
  if (!(val & O_NONBLOCK)) { val |= O_NONBLOCK; fcntl(fd, F_SETFL, val); }

  tcgetattr(fd, &oldtios); memcpy(&newtios, &oldtios, sizeof(oldtios));
  cfmakeraw(&newtios); tcsetattr(fd, TCSANOW, &newtios);
}

void reset_terminal(int fd)
{
  tcsetattr(fd, TCSANOW, &oldtios);
}



/*
 * Obtain a command from the user and return it.
 * Commands can come in from stdin only.
 */
int read_a_command(void)
{
  char ch;

  if (read(0, &ch, 1) != 1) return (CMD_NONE);  /* No kybrd data at present */

  switch (ch) {
    case ' ': return (CMD_PAUSE);
    case 'f': return (CMD_FWDFRAME);
    case 'b': return (CMD_BACKFRAME);
    case 3:                     /* ctrl-C and q */
    case 'Q':
    case 'q': return (CMD_QUIT);
    case 's': return (CMD_SPLITPOINT);
    case 'c': return (CMD_CSTART);
    case 'e': return (CMD_CEND);
    case 'E': return (CMD_CENDSPLIT);
    case 'R': return (CMD_RECORD);
    case 'P': return (CMD_PLAYITALL);
    case '1': return (CMD_FASTF1);
    case '2': return (CMD_FASTF2);
    case '3': return (CMD_FASTF3);
    case '4': return (CMD_FASTF4);
    case '5': return (CMD_FASTF5);
    case '6': return (CMD_FASTF6);
    case '!': return (CMD_FASTB1);
    case '@': return (CMD_FASTB2);
    case '#': return (CMD_FASTB3);
    case '$': return (CMD_FASTB4);
    case '%': return (CMD_FASTB5);
    case '^': return (CMD_FASTB6);
    case '0': return (CMD_END);
    case ')': return (CMD_START);
    case 'k': return (CMD_KILLCUT);
    case 'K': return (CMD_KILLALL);
    case 'l': return (CMD_LISTCUTS);
    case 'v': return (CMD_TOGSEECUT);
  }
  return (CMD_NONE);
}
